const fs = require('fs');
const path = require('path');

// Use __dirname to ensure the path is relative to the script's location
const typesDir = path.join(__dirname, 'types');
const indexFile = path.join(__dirname, 'index.ts');

function getAllDtsFiles(dir, fileList = []) {
  fs.readdirSync(dir).forEach(file => {
    const filePath = path.join(dir, file);
    if (fs.statSync(filePath).isDirectory()) {
      getAllDtsFiles(filePath, fileList);
    } else if (path.extname(file) === '.d.ts') {
      fileList.push(filePath);
    }
  });
  return fileList;
}

function processFiles(files) {
  const exports = [];

  files.forEach(filePath => {
    let content = fs.readFileSync(filePath, 'utf8');
    content = content.replace(/type\s+/g, 'export type ');
    fs.writeFileSync(filePath, content);

    const relativePath = path.relative(__dirname, filePath).replace(/\\/g, '/').replace('.d.ts', '');
    const match = content.match(/export type\s+(\w+)/g);
    if (match) {
      match.forEach(m => {
        const typeName = m.split(' ')[2];
        exports.push(`export type { ${typeName} } from "./${relativePath}";`);
      });
    }
  });

  return exports;
}

const allDtsFiles = getAllDtsFiles(typesDir);
const exportStatements = processFiles(allDtsFiles);

if (exportStatements.length > 0) {
  fs.writeFileSync(indexFile, exportStatements.join('\n') + '\n');
  console.log('Export statements have been added to index.ts');
} else {
  console.log('No export statements were generated.');
}
